@@ -29,8 +29,21 @@ from selenium import webdriver
 from robot.api import logger
 from robot.api.deco import keyword
 from robot.errors import RobotError
+from SeleniumLibrary import SeleniumLibrary
 from robot.libraries.BuiltIn import BuiltIn
 
+from robot.utils import (DotDict, escape, format_assign_message,
+                         get_error_message, get_time, html_escape, is_falsy, is_integer,
+                         is_string, is_truthy, is_unicode, IRONPYTHON, JYTHON,
+                         Matcher, normalize, NormalizedDict, parse_time, prepr,
+                         plural_or_not as s, PY3, RERAISED_EXCEPTIONS, roundup,
+                         secs_to_timestr, seq2str, split_from_equals, StringIO,
+                         timestr_to_secs, type_name, unic, is_list_like)
+from robot.errors import (ContinueForLoop, DataError, ExecutionFailed,
+                          ExecutionFailures, ExecutionPassed, ExitForLoop,
+                          PassExecution, ReturnFromKeyword)
+
+from selenium.common.exceptions import StaleElementReferenceException
 
 class Error(RuntimeError):
     ROBOT_CONTINUE_ON_FAILURE = True
@@ -116,6 +129,7 @@ class faciletoolkit:
         chrome_options.add_argument("--disable-extensions")
         chrome_options.add_argument("--disable-infobars")
         chrome_options.add_argument("--disable-browser-side-navigation")
+        chrome_options.add_argument("--ignore-certificate-errors")
         chrome_options.add_argument("--disable-dev-shm-usage")
         chrome_options.add_experimental_option("excludeSwitches", ["enable-automation"])
         chrome_options.set_capability("pageLoadStrategy", "eager")
@@ -171,12 +185,12 @@ class faciletoolkit:
 
     @staticmethod
     @keyword
-    def dom_is_loaded(timeout="5s"):
+    def dom_is_loaded(timeout="10s"):
         """ 
         returns True if DOM readyState equals "complete" before the given timeout.
         """
         driver = BuiltIn().get_library_instance('SeleniumLibrary')
-        driver.wait_for_condition("return document.readyState=='complete'", timeout)
+        BuiltIn().run_keyword('wait_for_condition', "return document.readyState=='complete'", timeout)
 
     @staticmethod
     @keyword
@@ -266,7 +280,7 @@ class faciletoolkit:
         """
         driver = BuiltIn().get_library_instance('SeleniumLibrary')
         driver.wait_for_condition("return document.location.href.includes('%s')" % url, timeout)
-        faciletoolkit().dom_is_loaded()
+        faciletoolkit().dom_is_loaded(timeout=timeout)
 
     @staticmethod
     @keyword
@@ -325,7 +339,12 @@ class faciletoolkit:
         driver = BuiltIn().get_library_instance('SeleniumLibrary')
         locator_option = "css:.autocomplete-suggestions:not([style*='display']):not([style*='none']) strong"
         BuiltIn().run_keyword_and_ignore_error("Checkpoint", locator_option, "2s")
-        BuiltIn().run_keyword_and_ignore_error("Javascript_click_ta", locator_option)
+        try:
+            return 'PASS', BuiltIn().run_keyword("faciletoolkit.wait", "Javascript_click_ta", locator_option)
+        except ExecutionFailed as err:
+            if err.dont_continue:
+                raise
+            return 'FAIL', unic(err)
 
     @staticmethod
     @keyword
@@ -343,8 +362,6 @@ class faciletoolkit:
             except Exception as error_thrown:
                 raise Error(error_thrown)
 
-            
-
     @staticmethod
     @keyword
     def press_key_on_active_element(key):
@@ -376,14 +393,32 @@ class faciletoolkit:
         Takes in input a WebElements array and
         returns a string array containing the stripped texts retrieved from the webelements
         """
-        driver = BuiltIn().get_library_instance('SeleniumLibrary')
         text_list = []
         for element in webelements:
-            text = driver.get_text(element)
+            text = faciletoolkit().get_text(element)
             text = text.strip()
             text_list.append(text)
         return text_list
 
+    @staticmethod
+    @keyword
+    def get_text(locator, max_attempts = 3):
+        """Overridden keyword get_text.
+        Returns the text value of element identified by `locator`.
+        Manage stale element exception with try/except for a maximum of 3 attempts
+        """
+        driver = BuiltIn().get_library_instance('SeleniumLibrary')
+        attempt = 1
+        while True:
+            try:
+                return  driver.find_element(locator).text
+            except StaleElementReferenceException:
+                logger.warn('Caught StaleElementReferenceException, re-trying the keyword')
+                if attempt == max_attempts:
+                    raise
+                attempt += 1
+                BuiltIn().run_keyword("Sleep", 2)
+
     @staticmethod
     @keyword
     def generate_random_number_as_string(minimum, maximum):
@@ -423,4 +458,16 @@ class faciletoolkit:
             faciletoolkit().checkpoint("//span[contains(.,'%s') and contains(@class,'err wrong')]" % message, "5s" )
             logger.info("%s is contained in error message" % message)
         except Exception as error_thrown:
-            raise Error(error_thrown)
\ No newline at end of file
+            raise Error(error_thrown)
+    
+    @staticmethod
+    @keyword
+    def previous_value_should_be_lower(values = []):
+        """
+        Performs a check on every value of a given list.
+        Raises an error if any previous value is higher than the actual iterator's value.
+        """
+        for previous, actual in zip(values, values[1:]):
+            if previous > actual:
+                raise(RobotError(str(previous) + " is bigger than " + str(actual)))
+
